import argparse
import numpy as np
from pydub import AudioSegment

def create_header(bit_depth, channels, sample_rate):
    # Create the first 6 bytes of the header (all 0xFF)
    header = bytearray([0xFF] * 6)

    # Create byte 7 with the specified bit settings
    byte7 = 0b00001000  # Bits 3, 2, 1, 0 are set to 1, 0, 0, 0

    if bit_depth == 16:
        byte7 |= 0b00100000  # Set bit 5 to 1 for 16 bit
    if channels == 2:
        byte7 |= 0b00010000  # Set bit 4 to 1 for stereo

    header.append(byte7)

    # Calculate byte 8 based on the sample rate with linear interpolation
    if 0 <= sample_rate <= 48828.125:
        byte8 = int(sample_rate / 48828.125 * 128)
    else:
        raise ValueError("Sample rate must be between 0Hz and 48828.125Hz.")

    header.append(byte8)

    return header

def convert_to_signed_pcm(data, bit_depth):
    if bit_depth == 8:
        # 8-bit PCM should remain signed
        return data
    elif bit_depth == 16:
        # 16-bit PCM should remain signed
        return data
    else:
        raise ValueError("Unsupported bit depth")

def convert_to_pcm(input_file, output_file, channels, bit_depth, sample_rate):
    # Load the audio file
    audio = AudioSegment.from_file(input_file)

    # Set the channels (mono or stereo)
    if channels == 1:
        audio = audio.set_channels(1)
    elif channels == 2:
        audio = audio.set_channels(2)
    else:
        raise ValueError("Channels must be 1 (mono) or 2 (stereo).")

    # Set the bit depth (8 or 16 bit)
    if bit_depth == 8:
        audio = audio.set_sample_width(1)
    elif bit_depth == 16:
        audio = audio.set_sample_width(2)
    else:
        raise ValueError("Bit depth must be 8 or 16.")

    # Set the sample rate
    if 0 <= sample_rate <= 48828.125:
        audio = audio.set_frame_rate(int(sample_rate))
    else:
        raise ValueError("Sample rate must be between 0Hz and 48828.125Hz.")

    # Export to raw PCM data
    pcm_data = audio.raw_data

    # Convert to signed PCM (if necessary, already signed)
    signed_pcm_data = convert_to_signed_pcm(pcm_data, bit_depth)

    # Create the custom header
    header = create_header(bit_depth, channels, sample_rate)

    # Write to output file with custom header
    with open(output_file, 'wb') as f:
        f.write(header)
        f.write(signed_pcm_data)

def main():
    parser = argparse.ArgumentParser(description='Convert audio file to PCM with custom settings.')
    parser.add_argument('input_file', type=str, help='Input audio file (wav or mp3).')
    parser.add_argument('output_file', type=str, help='Output PCM file.')
    parser.add_argument('--channels', type=int, choices=[1, 2], required=True, help='Number of channels: 1 (mono) or 2 (stereo).')
    parser.add_argument('--bit_depth', type=int, choices=[8, 16], required=True, help='Bit depth: 8 or 16.')
    parser.add_argument('--sample_rate', type=float, required=True, help='Sample rate between 0Hz and 48828.125Hz.')

    args = parser.parse_args()

    convert_to_pcm(args.input_file, args.output_file, args.channels, args.bit_depth, args.sample_rate)

if __name__ == '__main__':
    main()
